#include "star_packet.h"
#include <unistd.h>
#include <signal.h>
//#include <iconv.h>

const size_t msg_len = 1024;

char ifname[10] = ""; //网卡id
char username[100] = ""; //用户名
char password[100] = ""; //密码
char message[msg_len]; //服务器返回消息
uint8_t dest_mac[6]; //目标mac地址
uint16_t magic; 
uint16_t echo_key;
packet *net; //libpcap的packet类


/* 编码转换函数 */
/*
int convert( const char *desc, const char *src, char *input,
             size_t ilen, char *output, size_t olen) {
    char **pin = &input;
    char **pout = &output;
    iconv_t cd = iconv_open(desc, src);
    if (cd == (iconv_t)-1)
        return -1;
    memset(output, 0, olen);
    if (iconv(cd, pin, &ilen, pout, &olen)) return -1;
    iconv_close(cd);
    return 0;
}
*/

/* mac 地址打印函数 */
void print_mac(const char *msg, uint8_t *mac) {
    printf("%s: %02X %02X %02X %02X %02X %02X\n",
            msg, mac[0], mac[1], mac[2], mac[3], mac[4], mac[5], mac[6]);
}

/* 登出函数 */
/* 发送 EAPOL-Logoff packet */
void logoff(int signo) {
    printf("\n正在注销...\n");
    uint8_t *buf = new uint8_t[buf_len];
    net->send( (uint8_t *)packet_logoff(net->mac, dest_mac), 
               sizeof(eapol_packet));
    printf("成功注销:)\n");
    exit(0);
}

/* 认证函数 返回1表示认证成功,返回0表示认证失败,返回-1表示服务器应答不正确 */
/* 若认证成功或失败,message会设置成服务器返回的信息 */
/* 若认证成功, 服务器 mac 和 magic, echo_key 分别被设置 */
int auth(packet &net, char *username, char *password,
         char *message, uint8_t *dest_mac,
         uint16_t &magic, uint16_t &echo_key) {
    uint8_t *buf = new uint8_t[buf_len];
    uint8_t id;
    uint8_t key_len;
    uint8_t key[256];
    uint16_t _message_len;
    char _message[msg_len];
 
    eapol_packet *p;

    /* 发送 EAPOL-Start packet */
    printf("正在查找服务器... ");
    net.send( (uint8_t *)packet_start(net.mac, dest_mac),
              sizeof(eapol_packet));
    /* 接受返回 packet */
    if (net.recv(buf, buf_len) < 0)
        return -1;
    p = (eapol_packet *)buf;

    if (p->msg_type != SF_REQUEST) {
        /* 服务器返回错误 */
        return -1;
    }
    printf("找到了服务器\n");
    /* 保存服务器的 mac 地址 */
    memcpy(dest_mac, p->source_mac, 6);
    print_mac("服务器的mac地址: ", dest_mac);
    /* 保存 session id */
    id = p->id;

    /* 发送 EAPOL-Response username packet */
    printf("正在发送用户名... ");
    net.send((uint8_t *)packet_username(net.mac, dest_mac, id, username),
              sizeof(eapol_packet));
    /* 接受返回 packet */
    if (net.recv(buf, buf_len) < 0)
        return -1;
    p = (eapol_packet *)buf;
    if (p->msg_type != SF_REQUEST) {
        /* 服务器返回错误 */
        return -1;
    }
    printf("用户名OK\n");
    /* 保存 session id */
    id = p->id;
    /* 保存 key_len */
    key_len = p->body[0x00];
    /* 保存 key */
    memcpy(key, &(p->body[0x01]), key_len);

    /* 发送 EAPOL-Response password packet */
    printf("正在发送密码... ");
    net.send((uint8_t *)packet_password(net.mac, dest_mac, id, 
                                        username, password,
                                        key, key_len), 
             sizeof(eapol_packet));
    if (net.recv(buf, buf_len) < 0)
        return -1;
    p = (eapol_packet *)buf;
    if (p->msg_type == SF_SUCCESS) {
        /* 服务器返回 success */
        /* 保存 message */
        _message_len = ntohs(*((uint16_t *)(p->body + 0x03)));
        memcpy(_message, p->body + 0x05, _message_len);
        _message[_message_len] = 0;
        /* 编码转换 */
        //convert("UTF-8", "GBK", _message, msg_len, message, msg_len);
        /* 保存 magic 和 echo_key */
        uint16_t info_len = ntohs(*((uint16_t *)(p->body + _message_len + 0x09)));
        uint16_t skip = info_len + _message_len + 0x17;
        crypt(p->body + skip, 0x0F);
//        magic = ntohs(*((uint16_t *)(p->body + skip + 0x04)));
        magic = 0x102A;
        echo_key = ntohs(*((uint16_t *)(p->body + skip + 0x08)));
        return 1;
    }
    else if (p->msg_type == SF_FAILURE) {
        /* 服务器返回 failure */
        _message_len = ntohs(*((uint16_t *)(p->body + 0x03)));
        memcpy(_message, p->body + 0x05, _message_len);
        _message[_message_len] = 0;
        /* 编码转换 */
        //convert("UTF-8", "GBK", _message, msg_len, message, msg_len);
        return 0;
    }
    else {
        /* 服务器返回错误 */
        return -1;
    }
}

void help(){
    printf("用法: ./newstar -i 网卡id -u 用户名 -p 密码\n");
    printf("建议:   cp newstar /usr/bin\n");
    printf("        创建脚本/usr/bin/ruijie内容如下\n");
    printf("          #!/bin/bash\n");
    printf("          sudo echo\n");
    printf("          sudo setsid ./newstar -i 网卡id -u 你的用户名 -p 你的密码\n");
    printf("        修改属性 chmod +x /usr/bin/ruijie\n");
    printf("        以后只要在终端输入ruijie就可以上网拉，而且可以关掉那个终端:)\n");
    exit(0);
}

int main(int argc, char **argv) {
    printf("newstar 0.1 for ubuntu 8.10\n");
    printf("作者: Felicia @ WHUACM，修改自原superstar代码\n");
    printf("若发现错误，请联系: <feliciacrazy@qq.com>\n\n");

    
    /* 处理命令行参数 */
    int ch;
    opterr = 0;
    while ((ch = getopt(argc, argv, "i:u:p:")) != EOF) {
        switch (ch) {
            case 'i':
                strcpy(ifname, optarg);
                break;
            case 'u':
                strcpy(username, optarg);
                break;
            case 'p':
                strcpy(password, optarg);
                break;
            case '?':
                printf("无效的参数\n");
                help();
                break;
        }     
    }
    if (strcmp(ifname, "") == 0 || strcmp(username, "") == 0 || strcmp(password, "") == 0) {
        help();
        return 0;
    }

    net = new packet(ifname);

    /* 收到表示结束的信号时 logoff */
    signal(SIGHUP, logoff);  
    signal(SIGINT, logoff);
    signal(SIGQUIT, logoff);  
    signal(SIGABRT, logoff);
    signal(SIGKILL, logoff);  
    signal(SIGTERM, logoff);
    signal(SIGSTOP, logoff);  
    signal(SIGTSTP, logoff);

    //不断发送包裹向服务器请求连接
    while (1) {
        sleep(5);
        memcpy(dest_mac, broad_mac, sizeof(broad_mac));
        int ret = auth(*net, username, password, message, dest_mac, magic, echo_key);
        if (ret == -1) {
            printf("未能正确获得服务器的响应\n");
            continue;
        }
        else if (ret == 0) {
            printf("验证失败\n服务器消息: %s\n", message);
            continue;
        }
        printf("登录成功\n服务器消息: %s\n", message);
        break;
    }
    pid_t pid = getpid();
    printf("保持连接...\n");
    printf("按下Ctrl+C以注销(其他结束本进程(pid=%d)的方式亦可)\n", pid);
    fflush(stdout);
    //保持连接,每20s发送一次连接保持的数据包
    while (1) {
        net->send((uint8_t *)packet_echo(net->mac, dest_mac, magic, echo_key), 45);
        sleep(10);
    }
    return 0;
}
