#include "packet.h"

/* 初始化驱动, 打开socket, 获取网卡mac地址 */

packet::packet() {}

packet::packet(char *ifname) {
    sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_8021X));
    if (sock < 0) {
        perror("socket");
        return;
    }
    strncpy(req.ifr_name, ifname, IFNAMSIZ);
    int ioctl_ret = ioctl(sock, SIOGIFINDEX, &req);
    if (ioctl_ret == -1) {
        perror("set_device ioctl");
        return;
    }
    ifindex = req.ifr_ifindex;
    strncpy(this->ifname, ifname, IFNAMSIZ);
    dev.sll_family = AF_PACKET;
    dev.sll_protocol = ETH_P_8021X;
    dev.sll_ifindex = ifindex;
    dev.sll_halen = 6;

    ioctl_ret = ioctl(sock, SIOCGIFHWADDR, &req);
    if (ioctl_ret == -1) {
        perror("get_mac ioctl");
        return;
    }
    memcpy(mac, req.ifr_hwaddr.sa_data, IFHWADDRLEN);
}

/* 关闭socket */
packet::~packet() {
    close(sock);
}

/* 设置接收 packet 的超时时间 */
int packet::readable_time(int fd, int sec) {
    fd_set rset;
    struct timeval tv;
    FD_ZERO(&rset);
    FD_SET(fd, &rset);
    tv.tv_sec = sec;
    tv.tv_usec = 0;
    return (select(fd + 1, &rset, NULL, NULL, &tv));
}

/* 发送 packet */
int packet::send(uint8_t *buf, int buf_len) {
/*
	printf("sending packet:");
    printf("buf_len = %d\n", buf_len);
	for (int i = 0; i < buf_len; i++) {
		if (i % 16 == 0)
			printf("\n");
		printf("%02x ", buf[i]);
	}
	printf("\n");
    fflush(stdout);
*/
    int ret = sendto(sock, buf, buf_len, 0, (sockaddr *)&dev, sizeof(dev));
    if (ret < 0) perror("sendto");
    return ret;
}

/* 接收 pakcet */
int packet::recv(uint8_t *buf, int buf_len) {
    if (readable_time(sock, 2) == 0) {
        printf("Socket timeout\n");
        return -1;
    }
    else {
        socklen_t dev_size = sizeof(dev);
        int ret = recvfrom(sock, buf, buf_len, 0, (sockaddr *)&dev, &dev_size);
        if (ret < 0)
            perror("recvfrom");
/*
		printf("receive packet:");
		for (int i = 0; i < buf_len; i++) {
			if (i % 16 == 0)
				printf("\n");
			printf("%02x ", buf[i]);
		}
*/
        return ret;
    }
}

