#include "star_packet.h"


void fill_head(eapol_packet *packet, uint8_t *source, uint8_t *dest, uint8_t packet_type, uint16_t body_len_head) {
    memcpy(packet->dest_mac, dest, 6);
    memcpy(packet->source_mac, source, 6);
    packet->eap_type = htons(0x888E);
    packet->version = 0x01;
    packet->packet_type = packet_type;
    packet->body_len_head = htons(body_len_head);
}

void fill_auth(eapol_packet *packet, uint8_t id, uint16_t len, uint8_t auth_type) {
    packet->msg_type = SF_RESPONSE;                /* 总是response */
    packet->id = id;                               /* session id */
    packet->body_len = htons(len);                 /* 信息长度 */
    packet->auth_type = auth_type;                 /* IDENTITY or PASSWORD */
}

void fill_username(uint8_t *&p, char *username) {
    memcpy(p, username, strlen(username));
    p += strlen(username);
}

void fill_password(uint8_t *&p, uint8_t id, char *password, uint8_t *key, uint8_t key_len) {
    /* md5长度，总是0x10 */
    *(p++) = 0x10;                   
    uint8_t md5buf[256];
    uint8_t md5len;
    uint8_t *md5p = md5buf;
      
    /* 填充加密信息 */
    *(md5p++) = id;
    memcpy(md5p, password, strlen(password));
    md5p += strlen(password);
    memcpy(md5p, key, key_len);
    md5p += key_len;
    md5len = md5p - md5buf;
    md5p = ComputeHash(md5buf, md5len);

    /* 保存md5 */
    memcpy(p, md5p, 0x10);
    p += 0x10;
}

void crypt(uint8_t *p, int size) {
    unsigned char a,b,c,d;
    for (int i = 0; i < size; i++) {
        b = d = c = a = p[i];
        c &= 0x40;
        d >>= 2;
        c |= d;
        d = a;
        c >>= 2;
        d &= 0x20;
        c |= d;
        d = a;
        c >>= 2;
        d &= 0x10;
        c |= d;

        d = a;
        d &= 2;
        b <<= 2;
        d |= b;
        b = a;
        d <<= 2;
        b &= 4;
        a &= 8;
        d |= b;
        d <<= 2;
        d |= a;

        c >>= 1;
        d <<= 1;
        c |= d;
       
        c = ~c;
        p[i] = c;
    }
}

/* 加密算法居然没有变，和mystar是一样一样一样的:) */
void fill_param(uint8_t *&p) {
    unsigned char code[0x17];
    unsigned short ax,bx,dx;
    unsigned char icode[2];

    code[0] = 0x00;
    code[1] = 0x00;
    code[2] = 0x13;
    code[3] = 0x11;
    code[4] = 0x01;
    /*这个地方如果是1应该是实达加密或者是DHCP??*/
    *(uint32_t *)(code + 5) = 0;
    *(uint32_t *)(code + 9) = 0;
    *(uint32_t *)(code + 13) = 0;
    *(uint32_t *)(code + 17) = 0;
  
    ax = 0; bx = 0; dx = 0;
    for (int i = 0; i < 21; i++) {
        dx = ax;
        bx = 0;
        bx = (bx & 0xFF00) | code[i];
        dx &= 0xFFFF;
        dx >>= 8;
        dx ^= bx;
        bx = 0;
        bx &= 0x00FF;
        bx |= (ax & 0xFF) << 8;

        ax = table[dx * 2] | table[dx * 2 + 1] << 8;
        ax ^= bx;
    }
  
    code[0x15] = (unsigned char) ((ax & 0xFF00) >> 8);
    code[0x16] = (unsigned char) (ax & 0x00FF);

    crypt(code, 0x17);
    memcpy(p, code, 0x17);
    p += 0x17;

    memcpy(p, table2, sizeof(table2));
    p += sizeof(table2);
}

void fill_echo(uint8_t *p, uint16_t &magic, uint16_t &echo_key) {
    memcpy(p, table_echo, sizeof(table_echo));
    *((uint16_t *)(p + 0x08)) = htons(++magic + echo_key);
    *((uint16_t *)(p + 0x12)) = htons(magic);
    crypt(p, sizeof(table_echo));
}

eapol_packet *new_packet() {
    eapol_packet *packet = new eapol_packet;
    memset(packet, 0, sizeof(*packet));
    return packet;
}

eapol_packet *packet_start(uint8_t *source, uint8_t *dest) {
    eapol_packet *packet = new_packet();
    fill_head(packet, source, dest, EAP_START, 0x0000);
    return packet;
}

eapol_packet *packet_username(uint8_t *source, uint8_t *dest, uint8_t id, char *username) {
    eapol_packet *packet = new_packet();

    uint16_t len = strlen(username) + 0x05;

    fill_head(packet, source, dest, EAP_AUTH, len);
    fill_auth(packet, id, len, AUTH_IDENTITY);
    
    uint8_t *p = packet->body;
    fill_username(p, username);
    return packet;
}

eapol_packet *packet_password(uint8_t *source, uint8_t *dest, uint8_t id, char *username, char *password, uint8_t *key, uint8_t key_len) {
    eapol_packet *packet = new_packet();

    uint16_t len = strlen(username) + 0x16;

    fill_head(packet, source, dest, EAP_AUTH, len);
    fill_auth(packet, id, len, AUTH_PASSWORD);
    
    uint8_t *p = packet->body;
    fill_password(p, id, password, key, key_len);
    fill_username(p, username);
    fill_param(p);
    return packet;
}

eapol_packet *packet_echo(uint8_t *source, uint8_t *dest, uint16_t &magic, uint16_t echo_key) {
    eapol_packet *packet = new_packet();
    fill_head(packet, source, dest, EAP_ECHO, 0x1E);
    fill_echo(&(packet->msg_type), magic, echo_key);
    return packet;
}

eapol_packet *packet_logoff(uint8_t *source, uint8_t *dest) {
    eapol_packet *packet = new_packet();
    fill_head(packet, source, dest, EAP_LOGOFF, 0);
    return packet;
}

